#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-01-16
#
# Monitor status of Arista Networks BGP Peers (IPv4 and IPv6)
#
#
#

import copy
from typing import List, Dict, Optional, Tuple
from dataclasses import dataclass

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    startswith,
    OIDEnd,
    OIDBytes,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringByteTable,
)

from cmk.base.plugins.agent_based.utils.bgp_peer import (
    BgpPeer,
    bgp_get_peer_entry,
    bgp_get_ip_address_from_oid,
)

from cmk.base.plugins.agent_based.utils.arista_bgp_peer import (
    arista_afi_safi_mapping,
)


@dataclass
class AristaPrefixes:
    afi: str
    safi: str
    accepted_prefixes: int
    metric_count: List[Tuple[str, int]]
    metric_rate: List[Tuple[str, int]]


def parse_arista_bgp_peer(string_table: List[StringByteTable]) -> Optional[Dict[str, BgpPeer]]:
    BgpPeerTable, BgpPrefixCountersTable = string_table

    peer_table = {}
    peer_prefixes = {}

    for entry in BgpPrefixCountersTable:
        oid_end, in_prefixes, in_prefixes_accepted, out_prefixes = entry
        oid_end_split = oid_end.split('.')
        afi = oid_end_split[-2]
        safi = oid_end_split[-1]
        index = '.'.join(oid_end_split[:-2])
        prefixes = AristaPrefixes(
            afi=afi,
            safi=safi,
            accepted_prefixes=int(in_prefixes_accepted),
            metric_rate=[],
            metric_count=[
                ('in_prefixes', int(in_prefixes)),
                ('out_prefixes', int(out_prefixes)),
            ],
        )

        if peer_prefixes.get(index):
            peer_prefixes[index].append(prefixes)
        else:
            peer_prefixes[index] = [prefixes]

    for entry in BgpPeerTable:
        oid_end, remote_as, remote_id, admin_state, peer_state, description, in_updates, out_updates, \
        in_messages, out_messages, fsm_established_transitions, fsm_established_time, in_update_elapsed_time = entry

        remote_addr = bgp_get_ip_address_from_oid(f'{oid_end[2:]}')

        bgp_peer = bgp_get_peer_entry(
            remote_addr=remote_addr,
            peer_state=peer_state,
            admin_state=admin_state,
            description=description,
            remote_as=remote_as,
            in_updates=in_updates,
            out_updates=out_updates,
            in_messages=in_messages,
            out_messages=out_messages,
            fsm_established_transitions=fsm_established_transitions,
            fsm_established_time=fsm_established_time,
            in_update_elapsed_time=in_update_elapsed_time
        )

        prefixes = peer_prefixes.get(oid_end, [])
        for address_family in prefixes:
            address_family_str = arista_afi_safi_mapping(address_family.afi, address_family.safi)
            item = f'{remote_addr} {address_family_str}'
            peer_table[item] = copy.deepcopy(bgp_peer[remote_addr])
            peer_table[item].accepted_prefixes = address_family.accepted_prefixes
            peer_table[item].metric_count += address_family.metric_count
            peer_table[item].metric_rate += address_family.metric_rate
            peer_table[item].item['address_family'] = address_family_str
            peer_table[item].item['routing_instance'] = ''
    return peer_table


register.snmp_section(
    name='arista_bgp_peer',
    parse_function=parse_arista_bgp_peer,
    parsed_section_name='bgp_peer',
    supersedes=['bgp_peer', 'arista_bgp'],
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.30065.4.1.1',  # AARISTA-BGP4V2-MIB::aristaBgp4V2Objects
            oids=[
                OIDEnd(),
                '2.1.10',  # aristaBgp4V2PeerRemoteAs
                OIDBytes('2.1.11'),  # aristaBgp4V2PeerRemoteIdentifier
                '2.1.12',  # aristaBgp4V2PeerAdminStatus
                '2.1.13',  # aristaBgp4V2PeerState
                '2.1.14',  # aristaBgp4V2PeerDescription
                '7.1.1',  # aristaBgp4V2PeerInUpdates
                '7.1.2',  # aristaBgp4V2PeerOutUpdates
                '7.1.3',  # aristaBgp4V2PeerInTotalMessages
                '7.1.4',  # aristaBgp4V2PeerOutTotalMessages
                '7.1.5',  # aristaBgp4V2PeerFsmEstablishedTransitions
                '4.1.1',  # aristaBgp4V2PeerFsmEstablishedTime
                '4.1.2',  # aristaBgp4V2PeerInUpdatesElapsedTime
                ]),
        SNMPTree(
            base='.1.3.6.1.4.1.30065.4.1.1.8.1',  # ARISTA-BGP4V2-MIB::aristaBgp4V2PrefixGaugesEntry
            oids=[
                OIDEnd(),  # index.afi.safi
                '3',  # aristaBgp4V2PrefixInPrefixes
                '4',  # aristaBgp4V2PrefixInPrefixesAccepted
                '5',  # aristaBgp4V2PrefixOutPrefixes
            ]
        ),
    ],
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.30065'),  # ARISTA-SMI-MIB::arista
),
