#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2023-01-17
#
# inventory of Arista Networks BGP Peers (IPv4 and IPv6)
#
#


from typing import List
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    OIDBytes,
    startswith,
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringByteTable,
)
from cmk.base.plugins.agent_based.utils.bgp_peer import (
    InvBgpPeer,
    bgp_error_as_string,
    bgp_error_code_as_hex,
    bgp_render_ip_address,
    get_bgp_type,
    bgp_get_ip_address_from_oid,
)
from cmk.base.plugins.agent_based.utils.arista_bgp_peer import (
    arista_afi_safi_mapping,
)


def parse_inv_arista_bgp_peer(string_table: List[StringByteTable]):
    peers, address_families = string_table

    bgp_peers = []

    peer_families = {}
    for entry in address_families:
        oid_end, in_prefixes = entry
        oid_end_split = oid_end.split('.')
        afi = oid_end_split[-2]
        safi = oid_end_split[-1]
        index = '.'.join(oid_end_split[:-2])

        if peer_families.get(index):
            peer_families[index] = peer_families[index] + [arista_afi_safi_mapping(afi, safi)]
        else:
            peer_families[index] = [arista_afi_safi_mapping(afi, safi)]

    for entry in peers:
        oid_end, local_addr_type, local_addr, local_as, local_id, remote_as, remote_id, state, last_error_received, \
        fsm_established_time, fsm_established_trans = entry

        remote_addr = bgp_get_ip_address_from_oid(f'{oid_end[2:]}')

        # assuming IPv4 address format
        local_id = bgp_render_ip_address('1', local_id)
        remote_id = bgp_render_ip_address('1', remote_id)

        bgp_peer: InvBgpPeer = {
            'remote_addr': remote_addr,
            'remote_id': remote_id,
            'version': '4',
            'local_addr': bgp_render_ip_address(local_addr_type, local_addr),
            'remote_as': remote_as,
            'local_as': local_as,
            'local_id': local_id,
            'bgp_type': get_bgp_type(local_as, remote_as),
            'fsm_established_time': int(fsm_established_time),
            'peer_state': 1 if state == '6' else 2,  # adjust to match if_oper_status for inventory painter
            'last_error_code': bgp_error_code_as_hex(last_error_received),
            'last_error': bgp_error_as_string(last_error_received),
            'status_columns': {},
            'address_family': ', '.join(peer_families.get(oid_end, []))
        }
        bgp_peers.append(bgp_peer)

    return bgp_peers


register.snmp_section(
    name='inv_arista_bgp_peer',
    parse_function=parse_inv_arista_bgp_peer,
    parsed_section_name='inv_bgp_peer',
    supersedes=['inv_bgp_peer'],
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.30065.4.1.1',   # AARISTA-BGP4V2-MIB::aristaBgp4V2Objects
            oids=[
                OIDEnd(),
                '2.1.2',  # aristaBgp4V2PeerLocalAddrType
                OIDBytes('2.1.3'),  # aristaBgp4V2PeerLocalAddr
                '2.1.7',  # aristaBgp4V2PeerLocalAs
                OIDBytes('2.1.8'),  # aristaBgp4V2PeerLocalIdentifier
                '2.1.10',  # aristaBgp4V2PeerRemoteAs
                OIDBytes('2.1.11'),  # aristaBgp4V2PeerRemoteIdentifier
                '2.1.13',  # aristaBgp4V2PeerState
                OIDBytes('3.1.1'),  # aristaBgp4V2PeerLastErrorCodeReceived
                '4.1.1',  # aristaBgp4V2PeerFsmEstablishedTime
                '7.1.5',  # aristaBgp4V2PeerFsmEstablishedTransitions
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.30065.4.1.1.8.1',  # ARISTA-BGP4V2-MIB::aristaBgp4V2PrefixGaugesEntry
            oids=[
                OIDEnd(),  # index.addr-type.addr_length.addr.afi[-2].safi[-1]
                '3',  # aristaBgp4V2PrefixInPrefixes
            ]
        ),
    ],
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.30065'),  # ARISTA-SMI-MIB::arista
),
